****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_qrpe_createpov3.sas  
*
* Created (mm/dd/yyyy): 03/01/2018
* Last modified: 
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*	This program will identify all encounters meeting the inclusion/exclusion criteria    
*
*  Program inputs:                                                                                   
*	-Dataset with all encounters (ENC_&GROUP.) 
*	-Dataset with inclusion/exclusion codes
* 
*  Program outputs:                                                                                                                                       
* 	-Dataset with encounters meeting the inclusion/exclusion criteria (POV3)
* 
* PARAMETERS:
*  	
*
*  Programming Notes:                                                                                
*  -This macro evaluates whether each encounter, regardless of exposure, meets inclusion/exclusion
*	criteria. This is different from POV3 in QRP, which only evaluates whether exposure episodes
*	meet inclusion/exclusion criteria 
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials	   Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*
***************************************************************************************************;

%macro ms_qrpe_createpov3();

%put =====> MACRO CALLED: ms_qrpe_createpov3 v1.0;

	*Initialize for the case where no inclusion/exclusion criteria => keeping all encounters;
    data _POV3;
    set ENC_&GROUP.;
    keep Patid encounterID; 
    run;
    
	%let INCL=0;
    %let EXCL=0; 

	%ISDATA(dataset=&INCLUSIONCODES.);
    %IF %EVAL(&NOBS.>=1) %THEN %DO;

        *Are there any incl/excl conds to verify for this iteration?;
        data _IT&INCLUSIONCODES.;
        set &INCLUSIONCODES.;
        where group="&ITGROUP.";
        run;

        %ISDATA(dataset=_IT&INCLUSIONCODES.);
        %IF %EVAL(&NOBS.>=1) %THEN %DO;

            *Initialize list with all potential encounters;
            data _POV3;
	            set ENC_&GROUP. (rename=adate=enc_start rename=ddate=enc_end);
	            MustKickOutHard=0;  *cumulative;
	            keep PatId encounterID enc_start enc_end MustKickOut:;
            run;

            *Uniqueness of _InclExcl;
            proc sort nodupkey data=_InclExcl out=_InclExcl;
            	by Patid encounterID Adate Expiredt CondInclusion cond subcond SubCondInclusion;
            run;

            *Get all in INCL/EXCL period claims (both incl and Excl) 
             relative to each potential encounter;
            proc sql noprint;
            create table _POV3Claims as
            select distinct index.Patid,
                   			index.encounterID,
							index.enc_start,
							index.enc_end,
		                   	Inc.CondInclusion,
		                  	Inc.cond,
		                   	Inc.Subcond,
		                   	Inc.SubCondinclusion,
						   	Inc.codedays,
							Inc.gapdays,
						   	Inc.adate as incdate
            from  _POV3 as index,
                 _InclExcl as Inc
            where index.Patid = Inc.Patid and index.encounterID = inc.encounterID
            order by index.Patid, 
                     index.encounterID;
            quit;

            data _null_;
            	set _IT&INCLUSIONCODES.;
	            if CondInclusion=0 then call symputx("EXCL",1);
	            if CondInclusion=1 then call symputx("INCL",1);
            run;
            %put EXCL = &EXCL.;
            %put INCL = &INCL.; 

			*Determine how many conditions;
			%let MAXCOND=;
			proc sql;
				select max(cond) into: maxcond
				from _IT&INCLUSIONCODES.;
			quit;

			data _NULL_;
				call symputx("MAXCOND",&MAXCOND.);*For sum issue;
			run;
			%put MAXCOND = &MAXCOND.;

			%DO COND=1 %TO &MAXCOND.;

				%let MAXSUBCOND=0;
				*Determine how many levels of subconditions needed;
				proc sql;
				select max(subcond) into: maxsubcond
				from _IT&INCLUSIONCODES. (where=(cond=&COND.));
				quit;

				data _NULL_;
				call symputx("MAXSUBCOND",&MAXSUBCOND.);*For sum issue;
				run;
				%put MAXSUBCOND = &MAXSUBCOND.;

				*Determine whether this COND is an exclusion or inclusion;
				proc sql noprint;
					select max(CondInclusion) into: inclusion
					from _IT&INCLUSIONCODES. (where=(cond=&COND.)); 
				quit;
				%put &INCLUSION.;

				*Set data for this cond;
				data _POV3_&cond.;
					set _POV3(keep=Patid encounterID enc_start enc_end);
	                ExclCond&COND.=0;
				run;

				%DO SUBCOND=1 %TO &MAXSUBCOND.;

					%let subinclusion=0;
					*Determine if SUBCOND is an exclusion or inclusion;
					proc sql noprint;
						select max(SubCondInclusion) into: SUBCONDINCLUSION
						from _IT&INCLUSIONCODES. (where=(cond=&COND. and subcond=&SUBCOND.)); 

						select max(codedays) into: codedays
						from _IT&INCLUSIONCODES. (where=(cond=&COND. and subcond=&SUBCOND.)); 

						select max(gapdays) into: gapdays
						from _IT&INCLUSIONCODES. (where=(cond=&COND. and subcond=&SUBCOND.)); 
					quit;
					%put &SUBCONDINCLUSION. &codedays. &gapdays.;

					*Determine whether the apply Codedays alone or in combination with GapDays;
					%if %eval(&gapdays. >=0) %then %do;
						*Create episodes, bridging days where gap <= &gapdays. 
						 Take 1st episode and keep if length of episode >=codedays;

						proc sort data=_POV3Claims(where=(cond=&COND. and subcond=&SUBCOND.)) out=_POV3Incl_tmp;
							by patid encounterID incdate;
						run;

						data _POV3Incl_tmp;
							set _POV3Incl_tmp;
							by patid encounterID;

							if first.encounterID then do;
						    	LRunOutDate=incdate;
						      	episode=1;
						     	gap=.;
						   	end;
						   	else do;
						   		gap = incdate - LRunOutDate-1;     
 
								if gap > &gapdays. then do; 
									episode=episode+1; 
							   		LRunOutDate = incdate;		
							   	end;
							   	else do;
									LRunOutDate = incdate;
								end;
							end;

   							retain episode LRunOutDate;
						run;

						proc means data=_POV3Incl_tmp nway noprint missing;
							var incdate;
							class PatId EncounterID enc_start Episode;
							output out=_POV3Incl_tmp_epi(drop=_:) min(incdate)=IncStartdate
							                                  	max(incdate)=IncEndDate/KEEPLEN;
						run; 

						data _POV3Incl_tmp(where=(lengthepi>=&codedays.));
							set _POV3Incl_tmp_epi;
							by patid encounterID episode;
							if first.encounterID;

							lengthepi=(IncEndDate-IncStartDate)+1;
						run;

						proc sort data=_POV3Incl_tmp(drop=episode IncStartDate IncEndDate);
							by patid encounterID enc_start;
						run;
					%end;
					%else %do;
						*Do not create episodes. Only keep if codedays satisfied (number of dates >= codedays);
						proc means data=_POV3Claims noprint nway missing;
							class patid encounterID enc_start CondInclusion cond Subcond SubCondinclusion codedays / missing;
							output out=_POV3Incl_tmp(drop=_: where=(freq>=codedays)) n=freq;
							where cond=&COND. and subcond=&SUBCOND.;
						run;

			            proc sort nodupkey data=_POV3Incl_tmp out=_POV3Incl_tmp(keep=PatId encounterID enc_start);
				            by PatId encounterID enc_start;  *Here enc_start is of the episode, not the INCL;
				            where cond=&COND. and subcond=&SUBCOND.;
			            run;
					%end;

					*Square;						
					data _POV3_&cond.;
					merge _POV3_&cond.(in=a)
					      _POV3Incl_tmp(in=b);
					by PatId encounterID enc_start;

					if a and b then SubCond_&COND._&SUBCOND.=1;
					else SubCond_&COND._&SUBCOND.=0;

					if &SUBCONDINCLUSION.=1 then do;
						if a and b then SubCond_&COND._&SUBCOND.=1;
						else SubCond_&COND._&SUBCOND.=0;
					end;
					*If the subcondition is met but it is a subexclusion, then means that condition not satisfied;
					else do;
						if a and b then SubCond_&COND._&SUBCOND.=0;
						else SubCond_&COND._&SUBCOND.=1;
					end;
					run;
				%END; *loop subcond;
				
				*If all subconditions are satisfied, then condition is satisfied (at this point);
				data _POV3_&cond.;
					set _POV3_&cond.;
					Cond&cond.=sum(of SubCond_&COND._1-SubCond_&COND._&MAXSUBCOND.)=&MAXSUBCOND.;
				run;

        		*Manage if an exclusion - if you have one hard exclusion 
         		or not enough elig, you are out;
				data _POV3_&cond.;
				set _POV3_&cond.;
				if &INCLUSION.=0 then do;
	          	*if the patient had cond&cond.=1 then can never be included;
	          		if Cond&cond.=1 then MustKickOutHard_&cond.=1;
	          		else MustKickOutHard_&cond.=0;
	          		*flip the dummy;
	          		Cond&cond.=Cond&cond.=0;
	  				*Because we can have CondInclusion=0 and SubInclusion=1, we still need to consider if enough elig in this case;
				end;

				*Create a dummy to remember if this condition was an inclusion or and exlusion;
				InclCond&COND.=&INCLUSION.;
				run;

				*Put all together;
				data _POV3;
				merge _POV3(in=a)
				      _POV3_&cond.(in=b keep=PatId encounterID enc_start Cond: SubCond: Excl: InclCond: MustKickOut:);
				by PatId encounterID;
        		if MustKickOutHard_&cond.=1 then MustKickOutHard=1;
				if a;
				run;
			%END;*loop cond;

			*Clean up;
			data _POV3;															
			set _POV3;
      		array _cond(*) Cond1-cond&maxcond.;
      		array _InclCond(*) InclCond1-InclCond&maxcond.;

        	*At least one inclusion criterion;
	        %if &maxcond. >1 %then %do;
				inclusion=max(of InclCond1-InclCond&maxcond.);
			%end;
			%else %do;
	        	inclusion=InclCond1;
			%end;

	        *Initialize AnyCond;
	        if inclusion ne 1 then AnyCond=1;
	        else do;
	          AnyCond=0;
	          *Find those with at least one inclusion met;
	    			do i=1 to &maxcond.;
	            if _cond(i)=1 and _InclCond(i)=1 then AnyCond=1;
	    			end;	
	        end;
 
	        *Kick out any encounter that has at least one hard exclusion (claim).;
	        if MustKickOuthard then AnyCond=0;
	        drop inclusion;
		  	run;

			data _POV3Excl;	*for attrition;												
			set _POV3;
			if MustKickOutHard then output _POV3Excl;
			run;

			data _POV3Incl;	*for attrition;
			set _POV3;
			If AnyCond then output _POV3Incl;
			run;

			data _POV3;
			set _POV3Incl;
			run;

        %END; *_ITINCLUSIONCODES;
    %END; *INCLUSIONCODES;

%put NOTE: ********END OF MACRO: ms_qrpe_createpov3 v1.0********;

%mend ms_qrpe_createpov3;
